MODULE trcrst
   !!======================================================================
   !!                       ***  MODULE trcrst  ***
   !! TOP :   create, write, read the restart files for passive tracers
   !!======================================================================
   !! History :   1.0  !  2007-02 (C. Ethe) adaptation from the ocean
   !!----------------------------------------------------------------------
#if defined key_top
   !!----------------------------------------------------------------------
   !!   'key_top'                                                TOP models
   !!----------------------------------------------------------------------
   !!   trc_rst_opn    : open  restart file
   !!   trc_rst_read   : read  restart file
   !!   trc_rst_wri    : write restart file
   !!----------------------------------------------------------------------
   USE oce_trc
   USE trc
   USE sms_lobster         ! LOBSTER variables
   USE sms_pisces          ! PISCES variables
   USE trcsms_cfc          ! CFC variables
   USE trcsms_c14b         ! C14 variables
   USE trcsms_my_trc       ! MY_TRC variables
   USE trctrp_lec   
   USE lib_mpp
   USE iom
   
   IMPLICIT NONE
   PRIVATE
   
   PUBLIC   trc_rst_opn       ! called by ???
   PUBLIC   trc_rst_read      ! called by ???
   PUBLIC   trc_rst_wri       ! called by ???
   
   INTEGER, PUBLIC ::   numrtr, numrtw   !: logical unit for trc restart (read and write)


   !! * Substitutions
#  include "top_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/TOP 1.0 , LOCEAN-IPSL (2005) 
   !! $Id$ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
   
CONTAINS
   
   SUBROUTINE trc_rst_opn( kt )
      !!----------------------------------------------------------------------
      !!                    ***  trc_rst_opn  ***
      !!
      !! ** purpose  :   output of sea-trc variable in a netcdf file
      !!----------------------------------------------------------------------
      INTEGER, INTENT(in) ::   kt       ! number of iteration
      !
      CHARACTER(LEN=20)   ::   clkt     ! ocean time-step define as a character
      CHARACTER(LEN=50)   ::   clname   ! trc output restart file name
      !!----------------------------------------------------------------------
      !
      IF( kt == nit000 ) lrst_trc = .FALSE. 
# if defined key_off_tra
      IF( MOD( kt - 1, nstock ) == 0 ) THEN
         ! we use kt - 1 and not kt - nit000 to keep the same periodicity from the beginning of the experiment
         nitrst = kt + nstock - 1                  ! define the next value of nitrst for restart writing
         IF( nitrst > nitend )   nitrst = nitend   ! make sure we write a restart at the end of the run
      ENDIF
#endif
      
      IF( kt == nitrst - ndttrc .OR. nitend - nit000 + 1 < 2 * ndttrc ) THEN
         ! beware if model runs less than 2*ndttrc time step
         ! beware of the format used to write kt (default is i8.8, that should be large enough)
         IF( nitrst > 1.0e9 ) THEN   ;   WRITE(clkt,*       ) nitrst
         ELSE                        ;   WRITE(clkt,'(i8.8)') nitrst
         ENDIF
         ! create the file
         IF(lwp) WRITE(numout,*)
         clname = TRIM(cexper)//"_"//TRIM(ADJUSTL(clkt))//"_"//TRIM(cn_trcrst_out)
         IF(lwp) WRITE(numout,*) '             open trc restart.output NetCDF file: '//clname
         CALL iom_open( clname, numrtw, ldwrt = .TRUE., kiolib = jprstlib )
         lrst_trc = .TRUE.
      ENDIF
      !
   END SUBROUTINE trc_rst_opn


   SUBROUTINE trc_rst_read 
      !!----------------------------------------------------------------------
      !!                    ***  trc_rst_opn  ***
      !!
      !! ** purpose  :   read passive tracer fields in restart files
      !!----------------------------------------------------------------------
      INTEGER  ::  jn  
      INTEGER  ::  iarak0
      REAL(wp) ::  zarak0
#if defined key_pisces 
      INTEGER  ::  ji, jj, jk
      REAL(wp) ::  zcaralk, zbicarb, zco3
      REAL(wp) ::  ztmas, ztmas1
#endif

      !!----------------------------------------------------------------------

      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) 'trc_rst_read : read the TOP restart file'
      IF(lwp) WRITE(numout,*) '~~~~~~~~~~~~'

      CALL iom_open( cn_trcrst_in, numrtr, kiolib = jprstlib )

      ! Time domain : restart
      ! ---------------------
      CALL trc_rst_cal( nittrc000, 'READ' )   ! calendar

      IF( ln_trcadv_cen2 .OR. ln_trcadv_tvd ) THEN   ;   iarak0 = 1
      ELSE                                           ;   iarak0 = 0
      ENDIF
      CALL iom_get( numrtr, 'arak0', zarak0 )

      IF( iarak0 /= NINT( zarak0 ) )   &                           ! Control of the scheme
         & CALL ctl_stop( ' ===>>>> : problem with advection scheme', &
         & ' it must be the same type for both restart and previous run', &
         & ' centered or euler '  )
      IF(lwp) WRITE(numout,*)

      IF(lwp) WRITE(numout,*) '    arakawa option      : ', NINT( zarak0 )


      ! READ prognostic variables and computes diagnostic variable
      DO jn = 1, jptra
         CALL iom_get( numrtr, jpdom_autoglo, 'TRN'//ctrcnm(jn), trn(:,:,:,jn) ) 
      END DO

      DO jn = 1, jptra
         CALL iom_get( numrtr, jpdom_autoglo, 'TRB'//ctrcnm(jn), trb(:,:,:,jn) ) 
      END DO

#if defined key_lobster
      CALL iom_get( numrtr, jpdom_autoglo, 'SEDB'//ctrcnm(jpdet), sedpocb(:,:) ) 
      CALL iom_get( numrtr, jpdom_autoglo, 'SEDN'//ctrcnm(jpdet), sedpocn(:,:) ) 
#endif
#if defined key_pisces
      !
      IF( ln_pisdmp ) CALL pis_dmp_ini  ! relaxation of some tracers
      !
      IF( iom_varid( numrtr, 'PH', ldstop = .FALSE. ) > 0 ) THEN
         CALL iom_get( numrtr, jpdom_autoglo, 'PH' , hi(:,:,:)  )
      ELSE
         ! Set PH from  total alkalinity, borat (???), akb3 (???) and ak23 (???)
         ! --------------------------------------------------------
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji = 1, jpi
                  ztmas   = tmask(ji,jj,jk)
                  ztmas1  = 1. - tmask(ji,jj,jk)
                  zcaralk = trn(ji,jj,jk,jptal) - borat(ji,jj,jk) / (  1. + 1.E-8 / ( rtrn + akb3(ji,jj,jk) )  )
                  zco3    = ( zcaralk - trn(ji,jj,jk,jpdic) ) * ztmas + 0.5e-3 * ztmas1
                  zbicarb = ( 2. * trn(ji,jj,jk,jpdic) - zcaralk )
                  hi(ji,jj,jk) = ( ak23(ji,jj,jk) * zbicarb / zco3 ) * ztmas + 1.e-9 * ztmas1
               END DO
            END DO
         END DO
      ENDIF
      CALL iom_get( numrtr, jpdom_autoglo, 'Silicalim', xksi(:,:) ) 
      IF( iom_varid( numrtr, 'Silicamax', ldstop = .FALSE. ) > 0 ) THEN
         CALL iom_get( numrtr, jpdom_autoglo, 'Silicamax' , xksimax(:,:)  )
      ELSE
         xksimax(:,:) = xksi(:,:)
      ENDIF
#endif
#if defined key_cfc
      DO jn = jp_cfc0, jp_cfc1
         CALL iom_get( numrtr, jpdom_autoglo, 'qint_'//ctrcnm(jn), qint_cfc(:,:,jn) ) 
      END DO
#endif
#if defined key_c14b
      CALL iom_get( numrtr, jpdom_autoglo, 'qint_'//ctrcnm(jn) , qint_c14(:,:) ) 
#endif
#if defined key_my_trc
#endif
      
      CALL iom_close( numrtr )
      !
   END SUBROUTINE trc_rst_read


   SUBROUTINE trc_rst_wri( kt )
      !!----------------------------------------------------------------------
      !!                    ***  trc_rst_wri  ***
      !!
      !! ** purpose  :   write passive tracer fields in restart files
      !!----------------------------------------------------------------------
      INTEGER, INTENT( in ) ::   kt    ! ocean time-step index
      !!
      INTEGER  :: jn
      REAL(wp) :: zarak0
      !!----------------------------------------------------------------------


      CALL trc_rst_cal( kt, 'WRITE' )   ! calendar

      IF( ln_trcadv_cen2 .OR. ln_trcadv_tvd ) THEN   ;   zarak0 = 1.
      ELSE                                           ;   zarak0 = 0.
      ENDIF
      CALL iom_rstput( kt, nitrst, numrtw, 'arak0', zarak0 )

      ! prognostic variables
      ! --------------------
      DO jn = 1, jptra
         CALL iom_rstput( kt, nitrst, numrtw, 'TRN'//ctrcnm(jn), trn(:,:,:,jn) )
      END DO

      DO jn = 1, jptra
         CALL iom_rstput( kt, nitrst, numrtw, 'TRB'//ctrcnm(jn), trb(:,:,:,jn) )
      END DO

#if defined key_lobster
         CALL iom_rstput( kt, nitrst, numrtw, 'SEDB'//ctrcnm(jpdet), sedpocb(:,:) )
         CALL iom_rstput( kt, nitrst, numrtw, 'SEDN'//ctrcnm(jpdet), sedpocn(:,:) )
#endif
#if defined key_pisces 
         CALL iom_rstput( kt, nitrst, numrtw, 'PH', hi(:,:,:) )
         CALL iom_rstput( kt, nitrst, numrtw, 'Silicalim', xksi(:,:) )
         CALL iom_rstput( kt, nitrst, numrtw, 'Silicamax', xksimax(:,:) )
#endif
#if defined key_cfc
         DO jn = jp_cfc0, jp_cfc1
            CALL iom_rstput( kt, nitrst, numrtw, 'qint_'//ctrcnm(jn), qint_cfc(:,:,jn) )
         END DO
#endif
#if defined key_c14b
         CALL iom_rstput( kt, nitrst, numrtw, 'qint_'//ctrcnm(jn), qint_c14(:,:) )
#endif
#if defined key_my_trc
#endif
      
      IF( kt == nitrst ) THEN
          CALL trc_rst_stat            ! statistics
          CALL iom_close( numrtw )     ! close the restart file (only at last time step)
#if ! defined key_trdmld_trc
          lrst_trc = .FALSE.
#endif
      ENDIF
      !
   END SUBROUTINE trc_rst_wri

   SUBROUTINE trc_rst_cal( kt, cdrw )
      !!---------------------------------------------------------------------
      !!                   ***  ROUTINE trc_rst_cal  ***
      !!
      !!  ** Purpose : Read or write calendar in restart file:
      !!
      !!  WRITE(READ) mode:
      !!       kt        : number of time step since the begining of the experiment at the
      !!                   end of the current(previous) run
      !!       adatrj(0) : number of elapsed days since the begining of the experiment at the
      !!                   end of the current(previous) run (REAL -> keep fractions of day)
      !!       ndastp    : date at the end of the current(previous) run (coded as yyyymmdd integer)
      !!
      !!   According to namelist parameter nrstdt,
      !!       nrsttr = 0  no control on the date (nittrc000 is  arbitrary).
      !!       nrsttr = 1  we verify that nit000 is equal to the last
      !!                   time step of previous run + 1.
      !!       In both those options, the  exact duration of the experiment
      !!       since the beginning (cumulated duration of all previous restart runs)
      !!       is not stored in the restart and is assumed to be (nit000-1)*rdt.
      !!       This is valid is the time step has remained constant.
      !!
      !!       nrsttr = 2  the duration of the experiment in days (adatrj)
      !!                    has been stored in the restart file.
      !!----------------------------------------------------------------------
      INTEGER         , INTENT(in) ::   kt         ! ocean time-step
      CHARACTER(len=*), INTENT(in) ::   cdrw       ! "READ"/"WRITE" flag
      !
      REAL(wp) ::  zkt
#if defined key_off_tra
      REAL(wp) ::  zndastp
#endif

      ! Time domain : restart
      ! ---------------------

      IF( TRIM(cdrw) == 'READ' ) THEN
         CALL iom_get ( numrtr, 'kt', zkt )   ! last time-step of previous run
         IF(lwp) THEN
            WRITE(numout,*) ' *** Info read in restart : '
            WRITE(numout,*) '   previous time-step                               : ', NINT( zkt )
            WRITE(numout,*) ' *** restart option'
            SELECT CASE ( nrsttr )
            CASE ( 0 )   ;   WRITE(numout,*) ' nrsttr = 0 : no control of nittrc000'
            CASE ( 1 )   ;   WRITE(numout,*) ' nrsttr = 1 : no control the date at nit000 (use ndate0 read in the namelist)'
            CASE ( 2 )   ;   WRITE(numout,*) ' nrsttr = 2 : calendar parameters read in restart'
            END SELECT
            WRITE(numout,*)
         ENDIF
         ! Control of date 
         IF( nittrc000  - NINT( zkt ) /= 1 .AND.  nrsttr /= 0 )                                  &
            &   CALL ctl_stop( ' ===>>>> : problem with nit000 for the restart',                 &
            &                  ' verify the restart file or rerun with nrsttr = 0 (namelist)' )
#if defined key_off_tra
         ! define ndastp and adatrj
         IF ( nrsttr == 2 ) THEN
            CALL iom_get( numrtr, 'ndastp', zndastp ) 
            ndastp = NINT( zndastp )
            CALL iom_get( numrtr, 'adatrj', adatrj  )
         ELSE
            ndastp = ndate0 - 1     ! ndate0 read in the namelist in dom_nam
            adatrj = ( REAL( nittrc000-1, wp ) * rdttra(1) ) / rday
            ! note this is wrong if time step has changed during run
         ENDIF
         !
         IF(lwp) THEN
           WRITE(numout,*) ' *** Info used values : '
           WRITE(numout,*) '   date ndastp                                      : ', ndastp
           WRITE(numout,*) '   number of elapsed days since the begining of run : ', adatrj
           WRITE(numout,*)
         ENDIF
#endif

      ELSEIF( TRIM(cdrw) == 'WRITE' ) THEN
         !
         IF(  kt == nitrst ) THEN
            IF(lwp) WRITE(numout,*)
            IF(lwp) WRITE(numout,*) 'trc_wri : write the TOP restart file (NetCDF) at it= ', kt, ' date= ', ndastp
            IF(lwp) WRITE(numout,*) '~~~~~~~'
         ENDIF
         ! calendar control
         CALL iom_rstput( kt, nitrst, numrtw, 'kt'     , REAL( kt    , wp) )   ! time-step
         CALL iom_rstput( kt, nitrst, numrtw, 'ndastp' , REAL( ndastp, wp) )   ! date
         CALL iom_rstput( kt, nitrst, numrtw, 'adatrj' , adatrj            )   ! number of elapsed days since
         !                                                                     ! the begining of the run [s]
      ENDIF

   END SUBROUTINE trc_rst_cal

# if defined key_pisces 

   SUBROUTINE pis_dmp_ini 
      !!----------------------------------------------------------------------
      !!                    ***  pis_dmp_ini  ***
      !!
      !! ** purpose  : Relaxation of some tracers
      !!----------------------------------------------------------------------
      INTEGER  :: ji, jj, jk  
      REAL(wp) ::  &
         alkmean = 2426. ,  & ! mean value of alkalinity ( Glodap ; for Goyet 2391. )
         po4mean = 2.165 ,  & ! mean value of phosphates
         no3mean = 30.90 ,  & ! mean value of nitrate
         siomean = 91.51      ! mean value of silicate
      
      REAL(wp) ::   zvol, ztrasum


      IF(lwp)  WRITE(numout,*)

      IF( cp_cfg == "orca" .AND. .NOT. lk_trc_c1d ) THEN      ! ORCA condiguration (not 1D) !
         !                                                    ! --------------------------- !
         ! set total alkalinity, phosphate, NO3 & silicate

         ! total alkalinity
         ztrasum = 0.e0             
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji = 1, jpi
                  zvol = cvol(ji,jj,jk)
#  if defined key_off_degrad
                  zvol = zvol * facvol(ji,jj,jk)
#  endif
                  ztrasum = ztrasum + trn(ji,jj,jk,jptal) * zvol
               END DO
            END DO
         END DO
         IF( lk_mpp )   CALL mpp_sum( ztrasum )     ! sum over the global domain  
         
         ztrasum = ztrasum / areatot * 1.e6
         IF(lwp) WRITE(numout,*) '       TALK mean : ', ztrasum
         trn(:,:,:,jptal) = trn(:,:,:,jptal) * alkmean / ztrasum
            
         ! phosphate
         ztrasum = 0.e0
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji = 1, jpi
                  zvol = cvol(ji,jj,jk)
#  if defined key_off_degrad
                  zvol = zvol * facvol(ji,jj,jk)
#  endif
                  ztrasum = ztrasum + trn(ji,jj,jk,jppo4) * zvol
               END DO
            END DO
         END DO
         IF( lk_mpp )   CALL mpp_sum( ztrasum )     ! sum over the global domain  
         
         ztrasum = ztrasum / areatot * 1.e6 / 122.
         IF(lwp) WRITE(numout,*) '       PO4  mean : ', ztrasum
         trn(:,:,:,jppo4) = trn(:,:,:,jppo4) * po4mean / ztrasum
        
         ! Nitrates          
         ztrasum = 0.e0
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji = 1, jpi
                  zvol = cvol(ji,jj,jk)
#  if defined key_off_degrad
                  zvol = zvol * facvol(ji,jj,jk)
#  endif
                  ztrasum = ztrasum + trn(ji,jj,jk,jpno3) * zvol
               END DO
            END DO
         END DO
         IF( lk_mpp )   CALL mpp_sum( ztrasum )     ! sum over the global domain  
         
         ztrasum = ztrasum / areatot * 1.e6 / 7.6
         IF(lwp) WRITE(numout,*) '       NO3  mean : ', ztrasum
         trn(:,:,:,jpno3) = trn(:,:,:,jpno3) * no3mean / ztrasum
         
         ! Silicate
         ztrasum = 0.e0
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji = 1, jpi
                  zvol = cvol(ji,jj,jk)
#  if defined key_off_degrad
                  zvol = zvol * facvol(ji,jj,jk)
#  endif
                  ztrasum = ztrasum + trn(ji,jj,jk,jpsil) * zvol
               END DO
            END DO
         END DO
         IF( lk_mpp )   CALL mpp_sum( ztrasum )     ! sum over the global domain  
         ztrasum = ztrasum / areatot * 1.e6
         IF(lwp) WRITE(numout,*) '       SiO3 mean : ', ztrasum
         trn(:,:,:,jpsil) = MIN( 400.e-6,trn(:,:,:,jpsil) * siomean / ztrasum ) 
         !
      ENDIF

!#if defined key_kriest
!     !! Initialize number of particles from a standart restart file
!     !! The name of big organic particles jpgoc has been only change
!     !! and replace by jpnum but the values here are concentration
!     trn(:,:,:,jppoc) = trn(:,:,:,jppoc) + trn(:,:,:,jpnum)
!     trn(:,:,:,jpnum) = trn(:,:,:,jppoc) / ( 6. * xkr_massp )
!#endif

   END SUBROUTINE pis_dmp_ini

#endif
      !!----------------------------------------------------------------------

   SUBROUTINE trc_rst_stat
      !!----------------------------------------------------------------------
      !!                    ***  trc_rst_stat  ***
      !!
      !! ** purpose  :   Compute tracers statistics
      !!----------------------------------------------------------------------

      INTEGER  :: ji, jj, jk, jn
      REAL(wp) :: zdiag_var, zdiag_varmin, zdiag_varmax, zdiag_tot
      REAL(wp) :: zder, zvol
      !!----------------------------------------------------------------------


      IF( lwp ) THEN
         WRITE(numout,*) 
         WRITE(numout,*) '           ----TRACER STAT----             '
         WRITE(numout,*) 
      ENDIF
      
      zdiag_tot = 0.e0
      DO jn = 1, jptra
         zdiag_var    = 0.e0
         zdiag_varmin = 0.e0
         zdiag_varmax = 0.e0
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji = 1, jpi
                  zvol = cvol(ji,jj,jk)
#  if defined key_off_degrad
                  zvol = zvol * facvol(ji,jj,jk)
#  endif
                  zdiag_var = zdiag_var + trn(ji,jj,jk,jn) * zvol
               END DO
            END DO
         END DO
         
         zdiag_varmin = MINVAL( trn(:,:,:,jn), mask= ((tmask*SPREAD(tmask_i,DIM=3,NCOPIES=jpk).NE.0.)) )
         zdiag_varmax = MAXVAL( trn(:,:,:,jn), mask= ((tmask*SPREAD(tmask_i,DIM=3,NCOPIES=jpk).NE.0.)) )
         IF( lk_mpp ) THEN
            CALL mpp_min( zdiag_varmin )      ! min over the global domain
            CALL mpp_max( zdiag_varmax )      ! max over the global domain
            CALL mpp_sum( zdiag_var    )      ! sum over the global domain
         END IF
         zdiag_tot = zdiag_tot + zdiag_var
         zdiag_var = zdiag_var / areatot
         IF(lwp) WRITE(numout,*) '   MEAN NO ', jn, ctrcnm(jn), ' = ', zdiag_var,   &
            &                    ' MIN = ', zdiag_varmin, ' MAX = ', zdiag_varmax
      END DO
      
      zder = ( ( zdiag_tot - trai ) / ( trai + 1.e-12 )  ) * 100._wp
      IF(lwp) WRITE(numout,*) '   Integral of all tracers over the full domain  = ', zdiag_tot
      IF(lwp) WRITE(numout,*) '   Drift of the sum of all tracers =', zder, ' %'
      
   END SUBROUTINE trc_rst_stat

#else
   !!----------------------------------------------------------------------
   !!  Dummy module :                                     No passive tracer
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE trc_rst_read                      ! Empty routines
   END SUBROUTINE trc_rst_read
   SUBROUTINE trc_rst_wri( kt )
      INTEGER, INTENT ( in ) :: kt
      WRITE(*,*) 'trc_rst_wri: You should not have seen this print! error?', kt
   END SUBROUTINE trc_rst_wri   
#endif

   !!======================================================================
END MODULE trcrst
